@@ -4,8 +4,6 @@
 #ifndef SQUID_RADIX_H
 #define	SQUID_RADIX_H
 
-#include "config.h"
-
 /*
  * Copyright (c) 1988, 1989, 1993
  *      The Regents of the University of California.  All rights reserved.
@@ -113,36 +111,28 @@ struct squid_radix_node_head {
     int rnh_pktsize;		/* permit, but not require fixed keys */
 
     struct squid_radix_node *(*rnh_addaddr)	/* add based on sockaddr */
-                (void *v, void *mask,
-
-                 struct squid_radix_node_head * head, struct squid_radix_node nodes[]);
+    (void *v, void *mask, struct squid_radix_node_head * head, struct squid_radix_node nodes[]);
 
     struct squid_radix_node *(*rnh_addpkt)	/* add based on packet hdr */
-                (void *v, void *mask,
-
-                 struct squid_radix_node_head * head, struct squid_radix_node nodes[]);
+    (void *v, void *mask, struct squid_radix_node_head * head, struct squid_radix_node nodes[]);
 
     struct squid_radix_node *(*rnh_deladdr)	/* remove based on sockaddr */
-
-                (void *v, void *mask, struct squid_radix_node_head * head);
+    (void *v, void *mask, struct squid_radix_node_head * head);
 
     struct squid_radix_node *(*rnh_delpkt)	/* remove based on packet hdr */
-
-                (void *v, void *mask, struct squid_radix_node_head * head);
+    (void *v, void *mask, struct squid_radix_node_head * head);
 
     struct squid_radix_node *(*rnh_matchaddr)		/* locate based on sockaddr */
-
-                (void *v, struct squid_radix_node_head * head);
+    (void *v, struct squid_radix_node_head * head);
 
     struct squid_radix_node *(*rnh_lookup)	/* locate based on sockaddr */
 
-                (void *v, void *mask, struct squid_radix_node_head * head);
+    (void *v, void *mask, struct squid_radix_node_head * head);
 
     struct squid_radix_node *(*rnh_matchpkt)	/* locate based on packet hdr */
+    (void *v, struct squid_radix_node_head * head);
 
-                (void *v, struct squid_radix_node_head * head);
     int (*rnh_walktree)		/* traverse tree */
-
     (struct squid_radix_node_head * head, int (*f) (struct squid_radix_node *, void *), void *w);
 
     struct squid_radix_node rnh_nodes[3];	/* empty tree for common case */
