@@ -4,7 +4,6 @@
 class ACLChecklist;
 class ClientHttpRequest;
 
-#include "config.h"
 /* for RefCountable */
 #include "RefCount.h"
 /* for CBDATA_CLASS() */
@@ -37,16 +36,6 @@ public:
     void adaptationAccessCheck();
     void adaptationAclCheckDone(Adaptation::ServiceGroupPointer g);
 #endif
-#if USE_SSL
-    /**
-     * Initiates and start the acl checklist to check if the a CONNECT
-     * request must be bumped.
-     \retval true if the acl check scheduled, false if no ssl-bump required
-     */
-    bool sslBumpAccessCheck();
-    /// The callback function for ssl-bump access check list
-    void sslBumpAccessCheckDone(bool doSslBump);
-#endif
 
     ClientHttpRequest *http;
     ACLChecklist *acl_checklist;        /* need ptr back so we can unreg if needed */
@@ -60,9 +49,8 @@ public:
     bool redirect_done;
     bool no_cache_done;
     bool interpreted_req_hdrs;
-#if USE_SSL
-    bool sslBumpCheckDone;
-#endif
+    bool tosToClientDone;
+    bool nfmarkToClientDone;
 
 private:
     CBDATA_CLASS(ClientRequestContext);
