@@ -34,12 +34,14 @@
  */
 
 #include "squid.h"
-#include "CacheManager.h"
-#include "Store.h"
-#include "HttpHeader.h"
+#include "base64.h"
 #include "HttpHdrContRange.h"
 #include "HttpHdrSc.h"
+#include "HttpHeader.h"
 #include "MemBuf.h"
+#include "mgr/Registration.h"
+#include "rfc1123.h"
+#include "Store.h"
 
 /*
  * On naming conventions:
@@ -92,6 +94,7 @@ static const HttpHeaderFieldAttrs HeadersAttrs[] = {
     {"Content-Range", HDR_CONTENT_RANGE, ftPContRange},
     {"Content-Type", HDR_CONTENT_TYPE, ftStr},
     {"Cookie", HDR_COOKIE, ftStr},
+    {"Cookie2", HDR_COOKIE2, ftStr},
     {"Date", HDR_DATE, ftDate_1123},
     {"ETag", HDR_ETAG, ftETag},
     {"Expires", HDR_EXPIRES, ftDate_1123},
@@ -121,6 +124,7 @@ static const HttpHeaderFieldAttrs HeadersAttrs[] = {
     {"Retry-After", HDR_RETRY_AFTER, ftStr},	/* for now (ftDate_1123 or ftInt!) */
     {"Server", HDR_SERVER, ftStr},
     {"Set-Cookie", HDR_SET_COOKIE, ftStr},
+    {"Set-Cookie2", HDR_SET_COOKIE2, ftStr},
     {"TE", HDR_TE, ftStr},
     {"Title", HDR_TITLE, ftStr},
     {"Trailer", HDR_TRAILER, ftStr},
@@ -184,7 +188,7 @@ static http_hdr_type ListHeadersArr[] = {
     HDR_UPGRADE,
     HDR_VARY,
     HDR_VIA,
-    /* HDR_WARNING, */
+    HDR_WARNING,
     HDR_WWW_AUTHENTICATE,
     HDR_AUTHENTICATION_INFO,
     HDR_PROXY_AUTHENTICATION_INFO,
@@ -222,7 +226,7 @@ static http_hdr_type ReplyHeadersArr[] = {
     HDR_ACCEPT, HDR_ACCEPT_CHARSET, HDR_ACCEPT_ENCODING, HDR_ACCEPT_LANGUAGE,
     HDR_ACCEPT_RANGES, HDR_AGE,
     HDR_LOCATION, HDR_MAX_FORWARDS,
-    HDR_MIME_VERSION, HDR_PUBLIC, HDR_RETRY_AFTER, HDR_SERVER, HDR_SET_COOKIE,
+    HDR_MIME_VERSION, HDR_PUBLIC, HDR_RETRY_AFTER, HDR_SERVER, HDR_SET_COOKIE, HDR_SET_COOKIE2,
     HDR_VARY,
     HDR_WARNING, HDR_PROXY_CONNECTION, HDR_X_CACHE,
     HDR_X_CACHE_LOOKUP,
@@ -283,10 +287,9 @@ static void httpHeaderStatDump(const HttpHeaderStat * hs, StoreEntry * e);
 static void
 httpHeaderRegisterWithCacheManager(void)
 {
-    CacheManager::GetInstance()->
-    registerAction("http_headers",
-                   "HTTP Header Statistics",
-                   httpHeaderStoreReport, 0, 1);
+    Mgr::RegisterAction("http_headers",
+                        "HTTP Header Statistics",
+                        httpHeaderStoreReport, 0, 1);
 }
 
 void
@@ -1183,7 +1186,7 @@ HttpHeader::putSc(HttpHdrSc *sc)
     Packer p;
     assert(sc);
     /* remove old directives if any */
-    delById(HDR_RANGE);
+    delById(HDR_SURROGATE_CONTROL);
     /* pack into mb */
     mb.init();
     packerToMemInit(&p, &mb);
@@ -1195,6 +1198,14 @@ HttpHeader::putSc(HttpHdrSc *sc)
     mb.clean();
 }
 
+void
+HttpHeader::putWarning(const int code, const char *const text)
+{
+    char buf[512];
+    snprintf(buf, sizeof(buf), "%i %s \"%s\"", code, visible_appname_string, text);
+    putStr(HDR_WARNING, buf);
+}
+
 /* add extension header (these fields are not parsed/analyzed/joined, etc.) */
 void
 HttpHeader::putExt(const char *name, const char *value)
