@@ -4,6 +4,7 @@
 
 #include "squid.h"
 #include "comm.h"
+#include "comm/Write.h"
 #include "CommCalls.h"
 #include "HttpMsg.h"
 #include "adaptation/icap/Xaction.h"
@@ -18,6 +19,7 @@
 #include "icap_log.h"
 #include "fde.h"
 #include "SquidTime.h"
+#include "err_detail_type.h"
 
 static PconnPool *icapPconnPool = new PconnPool("ICAP Servers");
 
@@ -52,7 +54,6 @@ Adaptation::Icap::Xaction::~Xaction()
     debugs(93,3, typeName << " destructed, this=" << this <<
            " [icapx" << id << ']'); // we should not call virtual status() here
     HTTPMSGUNLOCK(icapRequest);
-    HTTPMSGUNLOCK(icapReply);
 }
 
 Adaptation::Icap::ServiceRep &
@@ -226,6 +227,7 @@ void Adaptation::Icap::Xaction::dieOnConnectionFailure()
 {
     debugs(93, 2, HERE << typeName <<
            " failed to connect to " << service().cfg().uri);
+    detailError(ERR_DETAIL_ICAP_XACT_START);
     throw TexcHere("cannot connect to the ICAP service");
 }
 
@@ -236,7 +238,7 @@ void Adaptation::Icap::Xaction::scheduleWrite(MemBuf &buf)
     writer = JobCallback(93,3,
                          Dialer, this, Adaptation::Icap::Xaction::noteCommWrote);
 
-    comm_write_mbuf(connection, &buf, writer);
+    Comm::Write(connection, &buf, writer);
     updateTimeout();
 }
 
@@ -269,7 +271,9 @@ void Adaptation::Icap::Xaction::handleCommTimedout()
            theService->cfg().methodStr() << " " <<
            theService->cfg().uri << status());
     reuseConnection = false;
-    throw TexcHere(connector != NULL ?
+    const bool whileConnecting = connector != NULL;
+    closeConnection(); // so that late Comm callbacks do not disturb bypass
+    throw TexcHere(whileConnecting ?
                    "timed out while connecting to the ICAP service" :
                    "timed out while talking to the ICAP service");
 }
@@ -283,6 +287,7 @@ void Adaptation::Icap::Xaction::noteCommClosed(const CommCloseCbParams &io)
 
 void Adaptation::Icap::Xaction::handleCommClosed()
 {
+    detailError(ERR_DETAIL_ICAP_XACT_CLOSE);
     mustStop("ICAP service connection externally closed");
 }
 
@@ -355,23 +360,31 @@ void Adaptation::Icap::Xaction::noteCommRead(const CommIoCbParams &io)
     Must(io.flag == COMM_OK);
     Must(io.size >= 0);
 
-    al.icap.bytesRead+=io.size;
+    if (!io.size) {
+        commEof = true;
+        reuseConnection = false;
 
-    updateTimeout();
+        // detect a pconn race condition: eof on the first pconn read
+        if (!al.icap.bytesRead && retriable()) {
+            setOutcome(xoRace);
+            mustStop("pconn race");
+            return;
+        }
+    } else {
 
-    debugs(93, 3, HERE << "read " << io.size << " bytes");
+        al.icap.bytesRead+=io.size;
 
-    /*
-     * See comments in Adaptation::Icap::Xaction.h about why we use commBuf
-     * here instead of reading directly into readBuf.buf.
-     */
+        updateTimeout();
+
+        debugs(93, 3, HERE << "read " << io.size << " bytes");
+
+        /*
+         * See comments in Adaptation::Icap::Xaction.h about why we use commBuf
+         * here instead of reading directly into readBuf.buf.
+         */
 
-    if (io.size > 0) {
         readBuf.append(commBuf, io.size);
         disableRetries(); // because pconn did not fail
-    } else {
-        reuseConnection = false;
-        commEof = true;
     }
 
     handleCommRead(io.size);
@@ -432,6 +445,7 @@ void Adaptation::Icap::Xaction::noteInitiatorAborted()
 
     if (theInitiator.set()) {
         clearInitiator();
+        detailError(ERR_DETAIL_ICAP_INIT_GONE);
         mustStop("initiator aborted");
     }
 
@@ -529,7 +543,7 @@ const char *Adaptation::Icap::Xaction::status() const
     buf.append("/", 1);
     fillDoneStatus(buf);
 
-    buf.Printf(" icapx%d]", id);
+    buf.Printf(" %s%u]", id.Prefix, id.value);
 
     buf.terminate();
 
