@@ -34,12 +34,14 @@
  */
 
 #include "squid.h"
+#include "comm/Write.h"
 #include "errorpage.h"
 #include "Store.h"
+#include "html_quote.h"
 #include "HttpRequest.h"
 #include "HttpReply.h"
 #include "comm.h"
-#if DELAY_POOLS
+#if USE_DELAY_POOLS
 #include "DelayPools.h"
 #include "MemObject.h"
 #endif
@@ -465,7 +467,7 @@ gopherToHTML(GopherStateData * gopherState, char *inbuf, int len)
                     len = TEMP_BUF_SIZE - gopherState->len;
                 }
 
-                xmemcpy(gopherState->buf + gopherState->len, inbuf, len);
+                memcpy(gopherState->buf + gopherState->len, inbuf, len);
                 gopherState->len += len;
                 return;
             }
@@ -496,7 +498,7 @@ gopherToHTML(GopherStateData * gopherState, char *inbuf, int len)
                 }
 
                 if (len > (pos - inbuf)) {
-                    xmemcpy(gopherState->buf, pos, len - (pos - inbuf));
+                    memcpy(gopherState->buf, pos, len - (pos - inbuf));
                     gopherState->len = len - (pos - inbuf);
                 }
 
@@ -775,8 +777,7 @@ gopherReadReply(int fd, char *buf, size_t len, comm_err_t flag, int xerrno, void
     int bin;
     size_t read_sz = BUFSIZ;
     int do_next_read = 0;
-#if DELAY_POOLS
-
+#if USE_DELAY_POOLS
     DelayId delayId = entry->mem_obj->mostBytesAllowed();
 #endif
 
@@ -794,15 +795,14 @@ gopherReadReply(int fd, char *buf, size_t len, comm_err_t flag, int xerrno, void
     }
 
     errno = 0;
-#if DELAY_POOLS
-
+#if USE_DELAY_POOLS
     read_sz = delayId.bytesWanted(1, read_sz);
 #endif
 
     /* leave one space for \0 in gopherToHTML */
 
     if (flag == COMM_OK && len > 0) {
-#if DELAY_POOLS
+#if USE_DELAY_POOLS
         delayId.bytesIn(len);
 #endif
 
@@ -984,7 +984,9 @@ gopherSendRequest(int fd, void *data)
     }
 
     debugs(10, 5, "gopherSendRequest: FD " << fd);
-    comm_write(fd, buf, strlen(buf), gopherSendComplete, gopherState, NULL);
+    AsyncCall::Pointer call = commCbCall(5,5, "gopherSendComplete",
+                                         CommIoCbPtrFun(gopherSendComplete, gopherState));
+    Comm::Write(fd, buf, strlen(buf), call, NULL);
 
     if (EBIT_TEST(gopherState->entry->flags, ENTRY_CACHABLE))
         gopherState->entry->setPublicKey();	/* Make it public */
