@@ -38,6 +38,7 @@
 #include "squid.h"
 #include "Store.h"
 #include "comm.h"
+#include "comm/Loops.h"
 #include "ICP.h"
 #include "HttpRequest.h"
 #include "acl/FilledChecklist.h"
@@ -85,6 +86,9 @@ static void icpHandleIcpV2(int, Ip::Address &, char *, int);
 /// \ingroup ServerProtocolICPInternal2
 static void icpCount(void *, int, size_t, int);
 
+/// \ingroup ServerProtocolICPInternal2
+static void icpGetOutgoingIpAddress();
+
 /**
  \ingroup ServerProtocolICPInternal2
  * IcpQueueHead is global so comm_incoming() knows whether or not
@@ -109,7 +113,7 @@ _icp_common_t::_icp_common_t(char *buf, unsigned int len)
         return;
     }
 
-    xmemcpy(this, buf, sizeof(icp_common_t));
+    memcpy(this, buf, sizeof(icp_common_t));
     /*
      * Convert network order sensitive fields
      */
@@ -235,15 +239,13 @@ void
 icpUdpSendQueue(int fd, void *unused)
 {
     icpUdpData *q;
-    int x;
-    int delay;
 
     while ((q = IcpQueueHead) != NULL) {
-        delay = tvSubUsec(q->queue_time, current_time);
+        int delay = tvSubUsec(q->queue_time, current_time);
         /* increment delay to prevent looping */
-        x = icpUdpSend(fd, q->address, (icp_common_t *) q->msg, q->logcode, ++delay);
+        const int x = icpUdpSend(fd, q->address, (icp_common_t *) q->msg, q->logcode, ++delay);
         IcpQueueHead = q->next;
-        safe_free(q);
+        xfree(q);
 
         if (x < 0)
             break;
@@ -290,7 +292,7 @@ _icp_common_t::createMessage(
     if (opcode == ICP_QUERY)
         urloffset += sizeof(uint32_t);
 
-    xmemcpy(urloffset, url, strlen(url));
+    memcpy(urloffset, url, strlen(url));
 
     return (icp_common_t *)buf;
 }
@@ -336,7 +338,7 @@ icpUdpSend(int fd,
             IcpQueueTail = queue;
         }
 
-        commSetSelect(fd, COMM_SELECT_WRITE, icpUdpSendQueue, NULL, 0);
+        Comm::SetSelect(fd, COMM_SELECT_WRITE, icpUdpSendQueue, NULL, 0);
         statCounter.icp.replies_queued++;
     } else {
         /* don't queue it */
@@ -615,7 +617,7 @@ icpHandleUdp(int sock, void *data)
     int len;
     int icp_version;
     int max = INCOMING_ICP_MAX;
-    commSetSelect(sock, COMM_SELECT_READ, icpHandleUdp, NULL, 0);
+    Comm::SetSelect(sock, COMM_SELECT_READ, icpHandleUdp, NULL, 0);
 
     while (max--) {
         len = comm_udp_recvfrom(sock,
@@ -678,9 +680,6 @@ icpConnectionsOpen(void)
     uint16_t port;
     Ip::Address addr;
 
-    struct addrinfo *xai = NULL;
-    int x;
-
     if ((port = Config.Port.icp) <= 0)
         return;
 
@@ -731,24 +730,23 @@ icpConnectionsOpen(void)
         if (theOutIcpConnection < 0)
             fatal("Cannot open Outgoing ICP Port");
 
-        commSetSelect(theOutIcpConnection,
-                      COMM_SELECT_READ,
-                      icpHandleUdp,
-                      NULL,
-                      0);
+        Comm::SetSelect(theOutIcpConnection, COMM_SELECT_READ, icpHandleUdp, NULL, 0);
 
         debugs(12, 1, "Outgoing ICP messages on port " << addr.GetPort() << ", FD " << theOutIcpConnection << ".");
 
         fd_note(theOutIcpConnection, "Outgoing ICP socket");
+        icpGetOutgoingIpAddress();
     }
+}
 
+static void
+icpGetOutgoingIpAddress()
+{
+    struct addrinfo *xai = NULL;
     theOutICPAddr.SetEmpty();
-
     theOutICPAddr.InitAddrInfo(xai);
 
-    x = getsockname(theOutIcpConnection, xai->ai_addr, &xai->ai_addrlen);
-
-    if (x < 0)
+    if (getsockname(theOutIcpConnection, xai->ai_addr, &xai->ai_addrlen) < 0)
         debugs(50, 1, "theOutIcpConnection FD " << theOutIcpConnection << ": getsockname: " << xstrerror());
     else
         theOutICPAddr = *xai;
@@ -764,11 +762,7 @@ icpIncomingConnectionOpened(int fd, int errNo, Ip::Address& addr)
     if (theInIcpConnection < 0)
         fatal("Cannot open ICP Port");
 
-    commSetSelect(theInIcpConnection,
-                  COMM_SELECT_READ,
-                  icpHandleUdp,
-                  NULL,
-                  0);
+    Comm::SetSelect(theInIcpConnection, COMM_SELECT_READ, icpHandleUdp, NULL, 0);
 
     for (const wordlist *s = Config.mcast_group_list; s; s = s->next)
         ipcache_nbgethostbyname(s->key, mcastJoinGroups, NULL);
@@ -777,8 +771,10 @@ icpIncomingConnectionOpened(int fd, int errNo, Ip::Address& addr)
 
     fd_note(theInIcpConnection, "Incoming ICP socket");
 
-    if (Config.Addrs.udp_outgoing.IsNoAddr())
+    if (Config.Addrs.udp_outgoing.IsNoAddr()) {
         theOutIcpConnection = theInIcpConnection;
+        icpGetOutgoingIpAddress();
+    }
 }
 
 /**
@@ -812,7 +808,7 @@ icpConnectionShutdown(void)
      */
     assert(theOutIcpConnection > -1);
 
-    commSetSelect(theOutIcpConnection, COMM_SELECT_READ, NULL, NULL, 0);
+    Comm::SetSelect(theOutIcpConnection, COMM_SELECT_READ, NULL, NULL, 0);
 }
 
 void
